@@ -43,6 +43,7 @@
 
 #include <libgnomeui/gnome-help.h>
 
+#include "gnome-settings-profile.h"
 #include "gsd-a11y-keyboard-manager.h"
 
 #define CONFIG_ROOT "/desktop/gnome/accessibility/keyboard"
@@ -56,6 +57,9 @@ struct GsdA11yKeyboardManagerPrivate
         gboolean   slowkeys_shortcut_val;
         GtkWidget *stickykeys_alert;
         GtkWidget *slowkeys_alert;
+
+        guint      gconf_notify;
+
 };
 
 #define GSD_KBD_A11Y_ERROR gsd_kbd_a11y_error_quark ()
@@ -74,7 +78,7 @@ static gpointer manager_object = NULL;
 
 #undef DEBUG_ACCESSIBILITY
 #ifdef DEBUG_ACCESSIBILITY
-#define d(str)          fprintf (stderr, str)
+#define d(str)          g_debug (str)
 #else
 #define d(str)          do { } while (0)
 #endif
@@ -88,15 +92,9 @@ gsd_kbd_a11y_error_quark (void)
 static gboolean
 xkb_enabled (GsdA11yKeyboardManager *manager)
 {
-        static gboolean initialized = 0;
-        static gboolean have_xkb = 0;
-
+        gboolean have_xkb;
         int opcode, errorBase, major, minor;
 
-        if (initialized) {
-                return have_xkb;
-        }
-
         gdk_error_trap_push ();
         have_xkb = XkbQueryExtension (GDK_DISPLAY (),
                                       &opcode,
@@ -115,10 +113,7 @@ static XkbDescRec *
 get_xkb_desc_rec (GsdA11yKeyboardManager *manager)
 {
         XkbDescRec *desc;
-        Status      status = Success; /* Any bogus value, to suppress warning */
-
-        if (! xkb_enabled (manager))
-                return NULL;
+        Status      status = Success;
 
         gdk_error_trap_push ();
         desc = XkbGetMap (GDK_DISPLAY (), XkbAllMapComponentsMask, XkbUseCoreKbd);
@@ -207,9 +202,10 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
         XkbDescRec      *desc;
         gboolean         enable_accessX;
 
+        gnome_settings_profile_start (NULL);
+
         desc = get_xkb_desc_rec (manager);
         if (!desc) {
-                d ("No XKB present\n");
                 return;
         }
 
@@ -314,8 +310,8 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
                                              XkbAccessXFeedbackMask | XkbAX_IndicatorFBMask);
 
         /*
-        fprintf (stderr, "CHANGE to : 0x%x\n", desc->ctrls->enabled_ctrls);
-        fprintf (stderr, "CHANGE to : 0x%x (2)\n", desc->ctrls->ax_options);
+        g_debug ("CHANGE to : 0x%x", desc->ctrls->enabled_ctrls);
+        g_debug ("CHANGE to : 0x%x (2)", desc->ctrls->ax_options);
         */
 
         gdk_error_trap_push ();
@@ -335,6 +331,8 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
 
         XSync (GDK_DISPLAY (), FALSE);
         gdk_error_trap_pop ();
+
+        gnome_settings_profile_end (NULL);
 }
 
 static gboolean
@@ -343,8 +341,7 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
                       guint                   revert_controls_mask,
                       gboolean                enabled)
 {
-        GError *err = NULL;
-        gboolean success;
+        GError *err;
 
         switch (response_id) {
         case GTK_RESPONSE_DELETE_EVENT:
@@ -357,20 +354,16 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
                 /* we're reverting, so we invert sense of 'enabled' flag */
                 d ("cancelling AccessX request");
                 if (revert_controls_mask == XkbStickyKeysMask) {
-                        success = gconf_client_set_bool (client,
-                                                         CONFIG_ROOT "/stickykeys_enable",
-                                                         !enabled,
-                                                         &err);
-                        if (err != NULL)
-                                g_error_free (err);
+                        gconf_client_set_bool (client,
+                                               CONFIG_ROOT "/stickykeys_enable",
+                                               !enabled,
+                                               NULL);
                 }
-                if (revert_controls_mask == XkbSlowKeysMask) {
-                        success = gconf_client_set_bool (client,
-                                                         CONFIG_ROOT "/slowkeys_enable",
-                                                         !enabled,
-                                                         &err);
-                        if (err != NULL)
-                                g_error_free (err);
+                else if (revert_controls_mask == XkbSlowKeysMask) {
+                        gconf_client_set_bool (client,
+                                               CONFIG_ROOT "/slowkeys_enable",
+                                               !enabled,
+                                               NULL);
                 }
                 gconf_client_suggest_sync (client, NULL);
                 set_server_from_gconf (manager, client);
@@ -380,6 +373,7 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
                 break;
         }
         case GTK_RESPONSE_HELP:
+                err = NULL;
                 gnome_help_display_desktop (NULL,
                                             "user-guide",
                                             "user-guide.xml",
@@ -392,8 +386,7 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
                                                                           GTK_BUTTONS_CLOSE,
                                                                           _("There was an error displaying help: %s"),
                                                                           err->message);
-                        g_signal_connect (G_OBJECT (error_dialog),
-                                          "response",
+                        g_signal_connect (error_dialog, "response",
                                           G_CALLBACK (gtk_widget_destroy), NULL);
                         gtk_window_set_resizable (GTK_WINDOW (error_dialog), FALSE);
                         gtk_widget_show (error_dialog);
@@ -464,7 +457,7 @@ ax_slowkeys_warning_dialog_post (GsdA11yKeyboardManager *manager,
         gtk_window_set_icon_name (GTK_WINDOW (manager->priv->slowkeys_alert),
                                   "gnome-dev-keyboard");
         gtk_dialog_set_default_response (GTK_DIALOG (manager->priv->slowkeys_alert),
-                                         GTK_RESPONSE_OK);
+                                         GTK_RESPONSE_ACCEPT);
 
         g_signal_connect (manager->priv->slowkeys_alert,
                           "response",
@@ -516,7 +509,7 @@ ax_stickykeys_warning_dialog_post (GsdA11yKeyboardManager *manager,
         gtk_window_set_icon_name (GTK_WINDOW (manager->priv->stickykeys_alert),
                                   "gnome-dev-keyboard");
         gtk_dialog_set_default_response (GTK_DIALOG (manager->priv->stickykeys_alert),
-                                         GTK_RESPONSE_OK);
+                                         GTK_RESPONSE_ACCEPT);
 
         g_signal_connect (manager->priv->stickykeys_alert,
                           "response",
@@ -541,7 +534,6 @@ set_gconf_from_server (GsdA11yKeyboardManager *manager)
         cs = gconf_change_set_new ();
         desc = get_xkb_desc_rec (manager);
         if (! desc) {
-                d ("No XKB present\n");
                 return;
         }
 
@@ -685,7 +677,7 @@ cb_xkb_event_filter (GdkXEvent              *xevent,
 
         if (xev->xany.type == (manager->priv->xkbEventBase + XkbEventCode) &&
             xkbEv->any.xkb_type == XkbControlsNotify) {
-                d ("Someone changed XKB state\n");
+                d ("XKB state changed");
                 set_gconf_from_server (manager);
         } else if (xev->xany.type == (manager->priv->xkbEventBase + XkbEventCode) &&
                    xkbEv->any.xkb_type == XkbAccessXNotify) {
@@ -713,17 +705,13 @@ keyboard_callback (GConfClient            *client,
 
 static void
 register_config_callback (GsdA11yKeyboardManager  *manager,
+                          GConfClient             *client,
                           const char              *path,
-                          GConfClientNotifyFunc    func)
+                          GConfClientNotifyFunc    func,
+                          guint                   *notify)
 {
-        GConfClient *client;
-
-        client = gconf_client_get_default ();
-
         gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
-        gconf_client_notify_add (client, path, func, manager, NULL, NULL);
-
-        g_object_unref (client);
+        *notify = gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
 gboolean
@@ -732,26 +720,31 @@ gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
 {
         guint        event_mask;
         GConfClient *client;
+        gboolean     ret = FALSE;
 
         g_debug ("Starting a11y_keyboard manager");
+        gnome_settings_profile_start (NULL);
+
+        if (!xkb_enabled (manager)) {
+                g_set_error (error, GSD_KBD_A11Y_ERROR,
+                             GSD_KBD_A11Y_ERROR_NOT_AVAILABLE,
+                             "XKB functionality is disabled.");
+                goto out;
+        }
+
+        client = gconf_client_get_default ();
 
         register_config_callback (manager,
+                                  client,
                                   CONFIG_ROOT,
-                                  (GConfClientNotifyFunc)keyboard_callback);
+                                  (GConfClientNotifyFunc) keyboard_callback,
+                                  &manager->priv->gconf_notify);
 
         event_mask = XkbControlsNotifyMask;
 #ifdef DEBUG_ACCESSIBILITY
         event_mask |= XkbAccessXNotifyMask; /* make default when AXN_AXKWarning works */
 #endif
 
-        if (!xkb_enabled (manager)) {
-                g_set_error (error, GSD_KBD_A11Y_ERROR,
-                             GSD_KBD_A11Y_ERROR_NOT_AVAILABLE,
-                             "XKB functionality is disabled.");
-                return FALSE;
-        }
-
-        client = gconf_client_get_default ();
         /* be sure to init before starting to monitor the server */
         set_server_from_gconf (manager, client);
         g_object_unref (client);
@@ -766,16 +759,41 @@ gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
         gdk_error_trap_pop ();
 
         gdk_window_add_filter (NULL,
-                               (GdkFilterFunc)cb_xkb_event_filter,
+                               (GdkFilterFunc) cb_xkb_event_filter,
                                manager);
-
-        return TRUE;
+        ret = TRUE;
+ out:
+        gnome_settings_profile_end (NULL);
+        return ret;
 }
 
 void
 gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
 {
+        GsdA11yKeyboardManagerPrivate *p = manager->priv;
+
         g_debug ("Stopping a11y_keyboard manager");
+
+        if (p->gconf_notify != 0) {
+                GConfClient *client = gconf_client_get_default ();
+                gconf_client_remove_dir (client, CONFIG_ROOT, NULL);
+                gconf_client_notify_remove (client, p->gconf_notify);
+                g_object_unref (client);
+                p->gconf_notify = 0;
+        }
+
+        gdk_window_remove_filter (NULL,
+                                  (GdkFilterFunc) cb_xkb_event_filter,
+                                  manager);
+
+        if (p->slowkeys_alert != NULL)
+                gtk_widget_destroy (p->slowkeys_alert);
+
+        if (p->stickykeys_alert != NULL)
+                gtk_widget_destroy (p->stickykeys_alert);
+
+        p->slowkeys_shortcut_val = FALSE;
+        p->stickykeys_shortcut_val = FALSE;
 }
 
 static void
@@ -842,7 +860,7 @@ gsd_a11y_keyboard_manager_dispose (GObject *object)
 static void
 gsd_a11y_keyboard_manager_class_init (GsdA11yKeyboardManagerClass *klass)
 {
-        GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+        GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->get_property = gsd_a11y_keyboard_manager_get_property;
         object_class->set_property = gsd_a11y_keyboard_manager_set_property;
@@ -857,7 +875,6 @@ static void
 gsd_a11y_keyboard_manager_init (GsdA11yKeyboardManager *manager)
 {
         manager->priv = GSD_A11Y_KEYBOARD_MANAGER_GET_PRIVATE (manager);
-
 }
 
 static void
