@@ -41,17 +41,21 @@
 #include <libgnomeui/gnome-bg.h>
 #include <X11/Xatom.h>
 
+#include "gnome-settings-profile.h"
 #include "gsd-background-manager.h"
 
 #include "preferences.h"
 
 #define GSD_BACKGROUND_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_BACKGROUND_MANAGER, GsdBackgroundManagerPrivate))
 
+#define GCONF_BG_DIR    "/desktop/gnome/background"
+
 struct GsdBackgroundManagerPrivate
 {
         BGPreferences *prefs;
         GnomeBG       *bg;
         guint          timeout_id;
+        guint          notify;
 };
 
 static void     gsd_background_manager_class_init  (GsdBackgroundManagerClass *klass);
@@ -156,6 +160,8 @@ nautilus_is_running (void)
 static gboolean
 apply_prefs (GsdBackgroundManager *manager)
 {
+        gnome_settings_profile_start (NULL);
+
         if (! nautilus_is_running ()) {
                 GdkDisplay      *display;
                 int              n_screens;
@@ -236,6 +242,8 @@ apply_prefs (GsdBackgroundManager *manager)
                 }
         }
 
+        gnome_settings_profile_end (NULL);
+
         return FALSE;
 }
 
@@ -275,6 +283,7 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
         gboolean nautilus_show_desktop;
 
         g_debug ("Starting background manager");
+        gnome_settings_profile_start (NULL);
 
         manager->priv->prefs = BG_PREFERENCES (bg_preferences_new ());
         manager->priv->bg = gnome_bg_new ();
@@ -285,15 +294,14 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
                           manager);
         bg_preferences_load (manager->priv->prefs);
 
-        apply_prefs (manager);
-
         client = gconf_client_get_default ();
-        gconf_client_notify_add (client,
-                                 "/desktop/gnome/background",
-                                 (GConfClientNotifyFunc)background_callback,
-                                 manager,
-                                 NULL,
-                                 NULL);
+        gconf_client_add_dir (client, GCONF_BG_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
+        manager->priv->notify = gconf_client_notify_add (client,
+                                                         GCONF_BG_DIR,
+                                                         (GConfClientNotifyFunc) background_callback,
+                                                         manager,
+                                                         NULL,
+                                                         NULL);
 
         /* If this is set, nautilus will draw the background and is
 	 * almost definitely in our session.  however, it may not be
@@ -302,13 +310,16 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
 	 * don't waste time setting the background only to have
 	 * nautilus overwrite it.
 	 */
-	nautilus_show_desktop = gconf_client_get_bool (client,
+        nautilus_show_desktop = gconf_client_get_bool (client,
                                                        "/apps/nautilus/preferences/show_desktop",
                                                        NULL);
         g_object_unref (client);
 
-        if (!nautilus_show_desktop)
+        if (!nautilus_show_desktop) {
                 apply_prefs (manager);
+        }
+
+        gnome_settings_profile_end (NULL);
 
         return TRUE;
 }
@@ -316,24 +327,39 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
 void
 gsd_background_manager_stop (GsdBackgroundManager *manager)
 {
+        GsdBackgroundManagerPrivate *p = manager->priv;
+
         g_debug ("Stopping background manager");
 
-        if (manager->priv->prefs != NULL) {
-        	g_object_unref (manager->priv->prefs);
-        	manager->priv->prefs = NULL;
+        if (p->notify != 0) {
+                GConfClient *client = gconf_client_get_default ();
+                gconf_client_remove_dir (client, GCONF_BG_DIR, NULL);
+                gconf_client_notify_remove (client, p->notify);
+                g_object_unref (client);
+                p->notify = 0;
         }
 
-        if (manager->priv->bg != NULL) {
-        	g_object_unref (manager->priv->bg);
-        	manager->priv->bg = NULL;
+        if (p->timeout_id != 0) {
+                g_source_remove (p->timeout_id);
+                p->timeout_id = 0;
+        }
+
+        if (p->prefs != NULL) {
+                g_object_unref (p->prefs);
+                p->prefs = NULL;
+        }
+
+        if (p->bg != NULL) {
+                g_object_unref (p->bg);
+                p->bg = NULL;
         }
 }
 
 static void
 gsd_background_manager_set_property (GObject        *object,
-                               guint           prop_id,
-                               const GValue   *value,
-                               GParamSpec     *pspec)
+                                     guint           prop_id,
+                                     const GValue   *value,
+                                     GParamSpec     *pspec)
 {
         GsdBackgroundManager *self;
 
@@ -348,9 +374,9 @@ gsd_background_manager_set_property (GObject        *object,
 
 static void
 gsd_background_manager_get_property (GObject        *object,
-                               guint           prop_id,
-                               GValue         *value,
-                               GParamSpec     *pspec)
+                                     guint           prop_id,
+                                     GValue         *value,
+                                     GParamSpec     *pspec)
 {
         GsdBackgroundManager *self;
 
@@ -365,8 +391,8 @@ gsd_background_manager_get_property (GObject        *object,
 
 static GObject *
 gsd_background_manager_constructor (GType                  type,
-                              guint                  n_construct_properties,
-                              GObjectConstructParam *construct_properties)
+                                    guint                  n_construct_properties,
+                                    GObjectConstructParam *construct_properties)
 {
         GsdBackgroundManager      *background_manager;
         GsdBackgroundManagerClass *klass;
